@@ -3,7 +3,7 @@
  *
  * See the COPYRIGHT file.
  *
- * $Id: acconfig.h,v 1.9 2000/07/31 22:33:54 kalt Exp $
+ * $Id: acconfig.h,v 1.11 2001/02/20 02:39:57 kalt Exp $
  *
  */
 
@@ -29,3 +29,9 @@
 
 /* Define if we're stuck with GNU Pth */
 #undef GNU_PTH
+
+/* PATH to yow */
+#define YOW_PATH ""
+
+/* Define if you have the getaddrinfo function.  */
+#undef HAVE_GETADDRINFO
