@@ -7,7 +7,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: commands.c,v 1.64 2000/07/31 22:33:54 kalt Exp $";
+static char rcsid[] = "@(#)$Id: commands.c,v 1.66 2001/03/14 19:50:20 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -30,6 +30,7 @@ extern int cmd_ping(char *);
 extern int cmd_seen(char *);
 extern int cmd_lastlog(char *);
 extern int sic_wreformat();
+extern char *yow();
 
 void parse_command(char *, int);
 
@@ -134,6 +135,7 @@ cmd_join(p)
   char *p;
 {
   char *key = index(p, ' ');
+  struct channel_ *ctmp;
 
   if (!*p)
       return -1;
@@ -142,7 +144,7 @@ cmd_join(p)
   if (server == NULL)
       return -5;
   if (key)
-      *key = '\0';
+      *key++ = '\0';
   switch (map_c2w(p))
     {
   case 0 :
@@ -158,6 +160,13 @@ cmd_join(p)
   default :
       abort(); /* never */
     }
+  ctmp = get_channel(p);
+  if (key && *key)
+    {
+      if (ctmp->chkey)
+	free(ctmp->chkey);
+      ctmp->chkey = strdup(key);
+    }
   join_channel(p, 0);
   return 0;
 }
@@ -762,6 +771,18 @@ cmd_window(p)
   return 0;
 }
 
+static int
+cmd_yow(p)
+char *p;
+{
+  char *zippy;
+
+  zippy = yow();
+  if (zippy)
+      vsic_slog(LOG_CLIENT, "--- %s", zippy);
+  return 0;
+}
+
 struct cmdlist_	cmdlist[] = {
 	{ "ADMIN",	NULL },
 	{ "ALIAS",	cmd_alias},
@@ -818,6 +839,7 @@ struct cmdlist_	cmdlist[] = {
 	{ "WHO",	cmd_who },
 	{ "WHOIS",	NULL },
 	{ "WHOWAS",	NULL },
+	{ "YOW",	cmd_yow },
 	{ NULL,		NULL }
 	};
 
