@@ -7,7 +7,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: display.c,v 1.57 2000/07/31 22:33:55 kalt Exp $";
+static char rcsid[] = "@(#)$Id: display.c,v 1.59 2001/03/14 19:50:20 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -27,6 +27,7 @@ extern char		tab_add(char *, char *);
 extern char		space_add(char *, char *);
 extern char		sic_ctcp(char *, char, char *);
 extern void		url_find(char *, char *, char *);
+extern void		yow_back(char *, char *);
 
 static char		outp[1024];
 static u_char		outa[1024];
@@ -314,7 +315,7 @@ sic_format(sender, cmd, dest, para)
 {
   int	fmt = -1;
   unsigned int flags = 0;
-  int	tocurrent = 1, ctcp = 0;
+  int	tocurrent = 1, ctcp = 0, yow = 0;
   char	tochannel[512];
 
   assert(server);
@@ -421,6 +422,8 @@ sic_format(sender, cmd, dest, para)
 	    }
 	  if (get_option(Z_URL, (toch) ? tochannel : NULL))
 	      url_find(sender, (toch) ? tochannel : NULL, p);
+	  if (toch && !froms && !fromv && !get_option(Z_NOYOW, tochannel))
+	      yow = 1;
 
 	  if (!froms && !toch)
 	      if (strncmp("\001DCC ", p, 5))
@@ -608,6 +611,18 @@ sic_format(sender, cmd, dest, para)
 		  *ch1 = '\0';
 		  if (map_c2w(p))
 		    {
+		      if (n == 475)
+			  {
+			    struct channel_ *ctmp;
+			    
+			    ctmp = get_channel(p);
+			    if (ctmp->chkey)
+			      {
+				vsic_slog(LOG_CLIENT, "--- Forgetting key \"%s\" for channel %s.", ctmp->chkey, p);
+				free(ctmp->chkey);
+				ctmp->chkey = NULL;
+			      }
+			  }
 		      if (get_option(Z_REJOIN, p))
 			  rejoin_channel(p);
 		      else
@@ -780,6 +795,8 @@ sic_format(sender, cmd, dest, para)
 
   if (ctcp)
       sic_ctcp(nick, toself, p);
+  if (yow)
+      yow_back(tochannel, p);
 }
 
 void
