@@ -7,7 +7,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: input.c,v 1.38 2000/07/31 22:33:55 kalt Exp $";
+static char rcsid[] = "@(#)$Id: input.c,v 1.40 2001/02/20 02:06:55 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -586,7 +586,7 @@ sic_getch()
 		      term_beep();
 		  break;
 	      case ctrl('T'):
-		  if (inpos)
+		  if (inpos && inpos < inlen)
 		    {
 		      char sw = inbuf[inpos];
 		      
@@ -607,6 +607,17 @@ sic_getch()
 	      case ctrl('V'):
 		  quote = 1;
 		  break;
+	      case ctrl('W'):
+		  if (inpos)
+		    {
+		      while (inpos && isspace((int) inbuf[inpos-1]))
+			  in_delprev();
+		      while (inpos && !isspace((int) inbuf[inpos-1]))
+			  in_delprev();
+		      chg = 1;
+		    }
+		  replace = 0;
+		  break;
 	      case ctrl('X'):
 		  screen = 1;
 		  break;
