@@ -1,7 +1,7 @@
 /*
  * os.h: getting the OS includes we need with the help of autoconf results
  *
- * Copyright(c) 1997,1998,1999 - All Rights Reserved
+ * Copyright(c) 1997-2000 - All Rights Reserved
  *
  * See the COPYRIGHT file.
  */
@@ -70,6 +70,20 @@
 # include <sys/select.h>
 #endif
 
+#include <netdb.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+
+#if !defined(INADDR_NONE)
+# define INADDR_NONE (unsigned long) -1
+#endif
+
+#ifdef BIND_HATRED
+# undef inet_ntoa
+# undef inet_addr
+#endif
+
 #if defined(UNSIGNED_LONG32) || defined(UNSIGNED_UNKNOWN32)
 typedef	unsigned long	u_32int_t;
 #endif
@@ -90,7 +104,21 @@ typedef	unsigned int	u_32int_t;
 #endif
 
 #if defined(__NetBSD__)
-# define PUTS_ARG_TYPE (void (*)(int))
+# include <sys/param.h>
+# if defined(__NetBSD_Version__) && (__NetBSD_Version__ < 104120000)
+#   define PUTS_ARG_TYPE (void (*)(int))
+# else
+	/* NetBSD's prototype for tputs is fixed as of 1.4K */
+#   define PUTS_ARG_TYPE
+# endif
 #else
 # define PUTS_ARG_TYPE
 #endif
+
+#if defined(HAVE_PTHREAD_H)
+# include <pthread.h>
+#endif
+
+#if defined(GNU_PTH)
+# include <pth.h>
+#endif
