@@ -7,7 +7,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: server.c,v 1.74 2000/08/01 20:07:34 kalt Exp $";
+static char rcsid[] = "@(#)$Id: server.c,v 1.83 2001/08/01 01:21:42 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -32,6 +32,37 @@ extern void sic_format(char *, char *, char *, char *);
 static struct server_	*slist = NULL;
 struct server_		*server = NULL;
 
+static int
+sic_nextaddr(srv)
+struct server_ *srv;
+{
+#if defined(HAVE_GETADDRINFO)
+  struct addrinfo *last;
+
+  assert(srv->address);
+  assert(srv->ip);
+
+  last = srv->addrip;
+  do
+    {
+      srv->addrip = srv->addrip->ai_next;
+      if (srv->addrip == NULL)
+	  srv->addrip = srv->address;
+    }
+  while (srv->addrip->ai_family != PF_INET &&
+	 srv->addrip->ai_family != PF_INET6);
+  free(srv->ip);
+  srv->ip = aitoip(srv->addrip);
+
+  vsic_slog(LOG_DEBUG, "sic_nextaddr: srv->addrip %x, last %x",
+	    srv->addrip, last);
+  if (srv->addrip != last)
+      return 1;
+#endif
+  vsic_slog(LOG_DEBUG, "sic_nextaddr returns 0");
+  return 0;
+}
+
 /* sic_connect: connect() to a server (given a struct server_ pointer) */
 void
 sic_connect(srv)
@@ -40,23 +71,16 @@ sic_connect(srv)
   struct  sockaddr_in	sock_sin;
   int			fd, opt = 1, rc;
   char			*vif;
-#if defined(HAVE_GETADDRINFO)
-  struct  sockaddr	sa;
 
-  if (srv->address)
-      bcopy((char *)srv->address->ai_addr, (char *)&sa,
-	    srv->address->ai_addrlen);
-#endif
-
-  assert (srv->ip);	/* even if srv->address is defined */
+  assert (srv->ip);	/* even if srv->addrip is defined */
 
   bzero((char *)&sock_sin, sizeof(sock_sin));
   sock_sin.sin_family = AF_INET;
 
 #if defined(HAVE_GETADDRINFO)
-  if (srv->address)
-      fd = socket(srv->address->ai_family, SOCK_STREAM,
-		  srv->address->ai_protocol);
+  if (srv->addrip)
+      fd = socket(srv->addrip->ai_family, SOCK_STREAM,
+		  srv->addrip->ai_protocol);
   else
 #endif
       fd = socket(AF_INET, SOCK_STREAM, 0);
@@ -79,8 +103,8 @@ sic_connect(srv)
 		srv->port, strerror(errno));
 
 #if defined(HAVE_GETADDRINFO)
-  if (srv->address)
-      rc = connect(fd, &sa, srv->address->ai_addrlen);
+  if (srv->addrip)
+      rc = connect(fd, srv->addrip->ai_addr, srv->addrip->ai_addrlen);
   else
 #endif
     {
@@ -91,27 +115,42 @@ sic_connect(srv)
 
   if (fd > 0 && rc < 0 && errno != EINPROGRESS)
     {
+      rc = errno;
       vsic_slog(LOG_CLIENT, "--- Connection to %s:%d failed: %s", srv->sname,
 		srv->port, strerror(errno));
       close(fd);
-      fd = -1;
-    }
-  if (fd == -1)
-    {
-      unset_option(srv->sopt, S_CONNECTING);
-      if (!option(srv->sopt, S_DCC)) /* needed? */
-	  srv->ts = time(NULL);
-    }
-  else
-    {
-      set_option(srv->sopt, S_CONNECTING);
-      
-      if (!option(srv->sopt, S_CONFIG))
-	  cfg_read(srv->sname);
-      set_option(srv->sopt, S_CONFIG);
+      if (rc == EHOSTUNREACH && !option(srv->sopt, S_DCC))
+	  fd = -2;
+      else
+	  fd = -1;
     }
   unset_option(srv->sopt, S_RECONNECT);
   unset_option(srv->sopt, S_QUIT);
+  switch (fd)
+    {
+    case -2: /* no route to host, may be there's another address to try? */
+	unset_option(srv->sopt, S_CONNECTING);
+	if (sic_nextaddr(srv))
+	  {
+	    set_option(srv->sopt, S_RECONNECT);
+	    srv->ts = time(NULL) - 4;
+	    break;
+	  }
+	srv->ts = time(NULL);
+	break;
+    case -1: /* other type of failures */
+	unset_option(srv->sopt, S_CONNECTING);
+	if (!option(srv->sopt, S_DCC)) /* needed? */
+	    srv->ts = time(NULL);
+	break;
+    default: /* connection in progress */
+	set_option(srv->sopt, S_CONNECTING);
+	
+	if (!option(srv->sopt, S_CONFIG))
+	    cfg_read(srv->sname);
+	set_option(srv->sopt, S_CONFIG);
+	break;
+    }
   srv->fd = fd;
   srv->lastr = 0;
   srv->readbuf[0] = '\0';
@@ -227,6 +266,7 @@ sic_server(nick, name, port, pass)
       {
 	server->ip = strdup(name);
 	server->address = address;
+	server->addrip = address;
       }
   }
 #else
@@ -253,15 +293,20 @@ sic_dserver(del)
 {
   struct server_ **stmp = &slist;
 
+  assert(sic_scnt(del) == 0);
+  assert(option(del->sopt, S_CONNECTED) == 0);
+  assert(option(del->sopt, S_DNS) == 0);
   while (*stmp != del)
       stmp = &((*stmp)->nexts);
-  /* pending DNS queries.. */
-  /* bound windows/channels.. */
   *stmp = del->nexts;
   opt_free(&(del->custs));
   free(del->sname);
   if (del->ip)
       free(del->ip);
+#if defined(HAVE_GETADDRINFO)
+  if (del->address)
+      freeaddrinfo(del->address);
+#endif
   if (del->pass)
       free(del->pass);
   free(del->nick);
@@ -411,7 +456,7 @@ int
 sic_select()
 {
   fd_set	rfd, wfd, efd;
-  int		n;
+  int		n, maxfd = 0;
   struct server_ *stmp = slist;
   struct timeval timeout;
   char		buf[10240];
@@ -422,6 +467,8 @@ sic_select()
   FD_SET(0, &rfd);
   while (stmp)
     {
+      if (stmp->fd > maxfd)
+	  maxfd = stmp->fd;
       server = stmp;
       if (option(stmp->sopt, S_NOTCONNECTED)
 	  && !option(stmp->sopt, S_CONNECTING)
@@ -478,9 +525,9 @@ sic_select()
   term_input(NULL, 0);
   term_flush();
 #if defined(GNU_PTH)
-  n = pth_select(255, &rfd, &wfd, NULL, &timeout);
+  n = pth_select(maxfd + 1, &rfd, &wfd, NULL, &timeout);
 #else
-  n = select(255, &rfd, &wfd, NULL, &timeout);
+  n = select(maxfd + 1, &rfd, &wfd, &efd, &timeout);
 #endif
 
   if (n < 0)
@@ -816,7 +863,7 @@ cmd_server(p)
   /* real user input */
   if (!*p || !strcasecmp(p, "-v"))
     {
-      /* no argument or just `-v', user is simply requestint a list */
+      /* no argument or just `-v', user is simply requesting a list */
       sic_slog(LOG_CLIENT, "--- Server list:");
       while (stmp)
 	{
@@ -831,19 +878,78 @@ cmd_server(p)
 				" Looking up address" : " Unknown address");
 	  if (*p && stmp->ip)
 #if defined(HAVE_GETADDRINFO)
+	    {
+	      struct addrinfo *address = stmp->address;
+
 	      vsic_slog(LOG_CLIENT, "---       => %s[%s]",
 			(stmp->address && stmp->address->ai_canonname) ?
 			stmp->address->ai_canonname : "???", stmp->ip);
+	      if (address && address->ai_next)
+		  while (address)
+		    {
+		      if (address->ai_family == PF_INET ||
+			  address->ai_family == PF_INET6)
+			  vsic_slog(LOG_CLIENT, "---        > %s[%s]",
+				    (address->ai_canonname) ?
+				    address->ai_canonname : "???",
+				    aitoip(address));
+		      address = address->ai_next;
+		    }
+	    }
 #else
 	      vsic_slog(LOG_CLIENT, "---       => [%s]", stmp->ip);
 #endif
 	  stmp = stmp->nexts;
 	}
     }
+  else if (!strncasecmp(p, "-d", 2))
+    {
+      char *number = p;
+
+      while (*number && !isdigit(*number))
+	  number += 1;
+      if (*number == '\0')
+	  return -4;
+
+      n = 0;
+      while (stmp)
+	{
+	  if (!option(stmp->sopt, S_DCC) && !option(stmp->sopt, S_LISTEN)
+	      && n++ == atoi(number))
+	      break;
+	  stmp = stmp->nexts;
+	}
+      if (stmp == NULL)
+	  return -4;
+
+      if (option(stmp->sopt, S_DNS))
+	{
+	  sic_slog(LOG_CLIENT,
+	   "--- DNS lookup in progress, cannot delete server. (try again)");
+	  return 0;
+	}
+      if (option(stmp->sopt, S_CONNECTED))
+	{
+	  sic_slog(LOG_CLIENT, "--- Close connection first.");
+	  return 0;
+	}
+      if (n = sic_scnt(stmp))
+	{
+	  vsic_slog(LOG_CLIENT,"--- %d window(s) are bound to this server.",n);
+	  return 0;
+	}
+      assert(server != stmp); /* this shouldn't be possible? */
+      vsic_slog(LOG_CLIENT,"--- Deleted server %d (%s).", atoi(number),
+		stmp->sname);
+      sic_dserver(stmp);
+      return 0;
+    }
   else
     {
       char *number = p;
 
+      if (*p == '+')
+	  number = p+1;
       while (*number)
 	  if (!isdigit(*number++))
 	    {
@@ -874,19 +980,19 @@ cmd_server(p)
 		{
 		  if (sic_swin(1))
 		    {
-		      if (option(stmp->sopt, S_CONNECTED))
-			{
-			  /*
-			  vsic_slog(LOG_CLIENT,
-				    "--- Window now bound to %s %d [%s]",
-				    stmp->sname, stmp->port, stmp->ip);
-			  */
-			}
-		      else
+		      if (!option(stmp->sopt, S_CONNECTED))
 			{
 			  set_option(stmp->sopt, S_CONNECTING);
-			  if (!option(stmp->sopt, S_DNS))
+			  if (option(stmp->sopt, S_DNS))
 			    {
+			      vsic_slog(LOG_CLIENT,
+				"--- Connection to %s pending DNS lookup.",
+					stmp->sname);
+			    }
+			  else
+			    {
+			      if (*p == '+')
+				  sic_nextaddr(stmp);
 			      vsic_slog(LOG_CLIENT,
 					"--- Connecting to %s %d [%s]",
 					stmp->sname, stmp->port, stmp->ip);
@@ -894,10 +1000,6 @@ cmd_server(p)
 			    }
 			}
 		    }
-		  /*
-		  else
-		      sic_slog(LOG_CLIENT, "--- No available window.");
-		  */
 		}
 	      else
 		  vsic_slog(LOG_CLIENT, "--- Unknown IP for %s", stmp->sname);
