@@ -1,13 +1,13 @@
 /*
  * term.c: all the routines we need to do our stuff with the screen.
  *
- * Copyright(c) 1997,1998,1999 - All Rights Reserved
+ * Copyright(c) 1997-2000 - All Rights Reserved
  *
  * See the COPYRIGHT file.
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: term.c,v 1.22 1999/08/02 22:20:34 kalt Exp $";
+static char rcsid[] = "@(#)$Id: term.c,v 1.24 2000/07/31 22:33:56 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -47,7 +47,8 @@ char    *CM,			/* screen-relative cursor motion */
 void
 term_init()
 {
-  char	termcap[1024], area[1024], *term, *ptr;
+  static char termcap[2048];
+  char	area[1024], *term, *ptr;
 
   int i = 0, missing = 0;
   char *cap_name[] = { "cm", "nd", "le", "ce", "cl", "dc", "ic", "so", "se", "us", "ue", "md", "me", "cs", "sf", "nl", "sr", "bl", 0 };
@@ -304,13 +305,16 @@ term_flush()
 }
 
 /* scroll n lines from line1 to line2 */
-void
+int
 term_scroll(line1, line2, n)
   int     line1, line2, n;
 {
   int     i;
   char    *code = NULL;
   
+  if (CS == NULL)
+    return -1;
+
   if (n > 0)
       code = SF ? SF : NL;
   else if (n < 0)
@@ -325,10 +329,12 @@ term_scroll(line1, line2, n)
       n = -n;
     }
   else
-      term_move_cursor(0, line2);
+    term_move_cursor(0, line2);
   for (i = 0; i < n; i++)
-      sic_tputs(code);
+    sic_tputs(code);
   sic_tputs(tgoto(CS, LI - 1, 0));
+
+  return 0;
 }
 
 void
