@@ -1,13 +1,13 @@
 /*
  * utils.c: various utility functions
  *
- * Copyright(c) 1997,1998,1999 - All Rights Reserved
+ * Copyright(c) 1997-2000 - All Rights Reserved
  *
  * See the COPYRIGHT file.
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: utils.c,v 1.12 1999/01/02 22:14:20 kalt Exp $";
+static char rcsid[] = "@(#)$Id: utils.c,v 1.14 2000/07/31 22:37:23 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -146,15 +146,15 @@ sic_tdiff(diff, force)
   return result[i];
 }
 
-#ifndef HAVE_STRERROR
+#if !defined(HAVE_STRERROR)
 
-# ifndef SYS_ERRLIST_DECLARED
+# if !defined(SYS_ERRLIST_DECLARED)
 extern char *sys_errlist[];
 # endif
 
-#ifndef SYS_NERR_DECLARED
+# if !defined(SYS_NERR_DECLARED)
 extern int sys_nerr;
-#endif
+# endif
 
 char *
 strerror(err_no)
@@ -173,3 +173,23 @@ strerror(err_no)
   return errp;
 }
 #endif
+
+#if defined(HAVE_GETADDRINFO)
+/* aitoip: return the IP address (string) contained in the addrinfo struct */
+char *
+aitoip(addr)
+struct addrinfo *addr;
+{
+    char tmp[512];
+
+    assert(addr->ai_family == AF_INET || addr->ai_family == AF_INET6);
+
+    if (inet_ntop(addr->ai_family, (addr->ai_family == AF_INET) ?
+		  &((struct sockaddr_in *)addr->ai_addr)->sin_addr :
+		  &((struct sockaddr_in6 *)addr->ai_addr)->sin6_addr,
+		  tmp, 512) == NULL)
+	abort();
+
+    return strdup(tmp);
+}
+#endif
