@@ -1,13 +1,11 @@
 /*
  * window.c: routines for the windows manipulations
  *
- * Copyright(c) 1997-2000 - All Rights Reserved
- *
  * See the COPYRIGHT file.
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: window.c,v 1.67 2000/07/31 23:28:28 kalt Exp $";
+static char rcsid[] = "@(#)$Id: window.c,v 1.69 2001/03/14 19:50:21 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -690,6 +688,22 @@ sic_swin(what)
   return cnt;
 }
 
+int
+sic_scnt(svr)
+struct server_ *svr;
+{
+  struct window_ *wtmp = wlist;
+  int i = 0;
+
+  while (wtmp)
+    {
+      if (wtmp->via == svr)
+	  i += 1;
+      wtmp = wtmp->nextw;
+    }
+  return i;
+}
+
 void
 sic_wkill()
 {
@@ -1271,6 +1285,7 @@ char *name;
   new->on->via = server;
   /*bzero(new->copt, sizeof(new->copt));*/
   new->chname = strdup(name);
+  new->chkey = NULL;
   save = current->defchan;
   current->defchan = new; /* so that cmd_option gets the right channel name */
   cfg_read(name);
@@ -1301,7 +1316,7 @@ join_channel(name, try)
 	  sic_slog(LOG_CLIENT, "--- Not connected.");
 	  return;
 	}
-      vsic_write("JOIN %s", name);
+      vsic_write("JOIN %s %s", name, (ctmp->chkey) ? ctmp->chkey : "");
       vsic_write("LAME:%s", name); /* trick to know when JOIN was completed */
       set_option(ctmp->copt, C_JOINING);
     }
@@ -1602,6 +1617,8 @@ collect_channel()
 	  *dtmp = del->nextc;
 	  
 	  free(del->chname);
+	  if (del->chkey)
+	      free(del->chkey);
 	  if ((w = del->on)->defchan == del)
 	    {
 	      struct channel_ *ctmp2 = clist;
